"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable functional/prefer-immutable-types */
const utils_1 = require("@typescript-eslint/utils");
const common_1 = require("./common");
/**
 * An ESLint rule to ban the partial URL construction.
 */
// eslint-disable-next-line total-functions/no-unsafe-readonly-mutable-assignment
const noPartialUrlConstructor = (0, common_1.createRule)({
    name: "no-partial-url-constructor",
    meta: {
        type: "problem",
        docs: {
            description: "Bans the partial URL construction.",
            recommended: "error",
        },
        messages: {
            errorStringGeneric: "Don't use the URL constructor directly because it can throw and because URLs are mutable. Instead, use `readonlyURL` from the readonly-types package.",
        },
        schema: [],
    },
    create: (context) => {
        return {
            // eslint-disable-next-line functional/no-return-void
            NewExpression: (node) => {
                // eslint-disable-next-line functional/no-conditional-statements
                if (node.callee.type === utils_1.AST_NODE_TYPES.Identifier &&
                    // TODO inspect the type instead of just relying on the type name.
                    node.callee.name === "URL") {
                    // eslint-disable-next-line functional/no-expression-statements
                    context.report({
                        node: node,
                        messageId: "errorStringGeneric",
                    });
                }
            },
        };
    },
    defaultOptions: [],
});
exports.default = noPartialUrlConstructor;
